==> sources_spiffyhud_fabric_3.1.0_MC_1.21.1.jar: de/keksuccino/spiffyhud/customization/elements/vanillalike/attackindicator/VanillaLikeAttackIndicatorElement.java <==
package de.keksuccino.spiffyhud.customization.elements.vanillalike.attackindicator;

import com.mojang.blaze3d.systems.RenderSystem;
import de.keksuccino.fancymenu.customization.element.AbstractElement;
import de.keksuccino.fancymenu.customization.element.ElementBuilder;
import de.keksuccino.fancymenu.util.rendering.RenderingUtils;
import net.minecraft.class_1309;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_332;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;

public class VanillaLikeAttackIndicatorElement extends AbstractElement {

    private static final Logger LOGGER = LogManager.getLogger();

    // Sprite resources for attack indicators in 1.21.1
    private static final class_2960 CROSSHAIR_ATTACK_INDICATOR_FULL_SPRITE = class_2960.method_60656("hud/crosshair_attack_indicator_full");
    private static final class_2960 CROSSHAIR_ATTACK_INDICATOR_BACKGROUND_SPRITE = class_2960.method_60656("hud/crosshair_attack_indicator_background");
    private static final class_2960 CROSSHAIR_ATTACK_INDICATOR_PROGRESS_SPRITE = class_2960.method_60656("hud/crosshair_attack_indicator_progress");
    private static final class_2960 HOTBAR_ATTACK_INDICATOR_BACKGROUND_SPRITE = class_2960.method_60656("hud/hotbar_attack_indicator_background");
    private static final class_2960 HOTBAR_ATTACK_INDICATOR_PROGRESS_SPRITE = class_2960.method_60656("hud/hotbar_attack_indicator_progress");

    private final class_310 minecraft = class_310.method_1551();

    public boolean isHotbar = false;

    public VanillaLikeAttackIndicatorElement(@NotNull ElementBuilder<?, ?> builder) {
        super(builder);
    }

    /**
     * Renders the experience bar within this element's bounds.
     *
     * @param graphics The graphics context.
     * @param mouseX   The current mouse X-coordinate.
     * @param mouseY   The current mouse Y-coordinate.
     * @param partial  Partial ticks.
     */
    @Override
    public void method_25394(@NotNull class_332 graphics, int mouseX, int mouseY, float partial) {

        // Do nothing if the player or level is missing.
        if (this.minecraft.field_1724 == null || this.minecraft.field_1687 == null) {
            return;
        }

        // Get the absolute position and dimensions of this element.
        int elementX = this.getAbsoluteX();
        int elementY = this.getAbsoluteY();
        int elementWidth = this.getAbsoluteWidth();
        int elementHeight = this.getAbsoluteHeight();

        // Prepare the render state.
        RenderSystem.enableBlend();
        RenderingUtils.resetShaderColor(graphics);

        // Render the experience bar using the element's own bounds.
        this.renderAttackIndicator(graphics, this.isHotbar, elementX, elementY, elementWidth, elementHeight);

        RenderingUtils.resetShaderColor(graphics);

    }

    /**
     * Renders the attack indicator in either crosshair or hotbar mode, scaled to the specified dimensions.
     *
     * @param graphics the GuiGraphics object used for rendering
     * @param isHotbar whether to render the hotbar version (true) or crosshair version (false)
     * @param x the x position to render at
     * @param y the y position to render at
     * @param width the width to render the indicator at
     * @param height the height to render the indicator at
     */
    public void renderAttackIndicator(class_332 graphics, boolean isHotbar, int x, int y, int width, int height) {

        float attackStrength = this.minecraft.field_1724.method_7261(0.0f);
        if (isEditor()) attackStrength = 0.5f;

        RenderSystem.enableBlend();
        graphics.method_51422(1.0F, 1.0F, 1.0F, this.opacity);

        if (isHotbar) {
            // Render hotbar version (only shown when not fully charged)
            if (attackStrength < 1.0f) {
                // First render the background
                graphics.method_52706(HOTBAR_ATTACK_INDICATOR_BACKGROUND_SPRITE, x, y, width, height);

                // Then render the foreground based on attack strength
                int fillHeight = (int)(height * attackStrength);
                if (fillHeight > 0) {
                    // For hotbar indicator, the original sprite is 18x18 and fills from bottom
                    graphics.method_52708(HOTBAR_ATTACK_INDICATOR_PROGRESS_SPRITE, 
                        width, height,  // Full widget size
                        0, height - fillHeight,  // Start from the bottom
                        x, y + height - fillHeight,  // Position at the bottom
                        width, fillHeight  // Fill portion from bottom
                    );
                }
            }
        } else {
            // Render crosshair version
            boolean readyToAttack = false;
            if (this.minecraft.field_1692 != null &&
                    this.minecraft.field_1692 instanceof class_1309 &&
                    attackStrength >= 1.0f) {
                readyToAttack = this.minecraft.field_1724.method_7279() > 5.0f;
                readyToAttack &= this.minecraft.field_1692.method_5805();
            }

            if (readyToAttack) {
                // Render the "ready to attack" indicator
                graphics.method_52706(CROSSHAIR_ATTACK_INDICATOR_FULL_SPRITE, x, y, width, height);
            } else if (attackStrength < 1.0f) {
                // Determine a proper height for the progress bar (maintaining aspect ratio)
                int barHeight = height;
                if (width / height > 4) { // Original aspect ratio is 16:4 or 4:1
                    barHeight = Math.max(1, width / 4);
                }

                // First render the background bar
                graphics.method_52706(CROSSHAIR_ATTACK_INDICATOR_BACKGROUND_SPRITE, x, y, width, barHeight);

                // Then render the foreground based on attack strength
                int fillWidth = (int)(width * attackStrength);
                if (fillWidth > 0) {
                    // For crosshair indicator, the original sprite is 16x4 and fills from left
                    graphics.method_52708(CROSSHAIR_ATTACK_INDICATOR_PROGRESS_SPRITE, 
                        width, barHeight,  // Full widget size
                        0, 0,  // Start from top-left
                        x, y,  // Position at top-left
                        fillWidth, barHeight  // Fill portion from left
                    );
                }
            }
        }

        graphics.method_51422(1.0F, 1.0F, 1.0F, 1.0F);

    }

}

==> sources_spiffyhud_fabric_3.1.1_MC_1.19.2.jar: de/keksuccino/spiffyhud/customization/elements/vanillalike/attackindicator/VanillaLikeAttackIndicatorElement.java <==
package de.keksuccino.spiffyhud.customization.elements.vanillalike.attackindicator;

import com.mojang.blaze3d.systems.RenderSystem;
import de.keksuccino.fancymenu.customization.element.AbstractElement;
import de.keksuccino.fancymenu.customization.element.ElementBuilder;
import de.keksuccino.fancymenu.util.rendering.RenderingUtils;
import de.keksuccino.fancymenu.util.rendering.gui.GuiGraphics;
import net.minecraft.class_1309;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;

public class VanillaLikeAttackIndicatorElement extends AbstractElement {

    private static final Logger LOGGER = LogManager.getLogger();

    // Texture used for drawing the experience bar icons.
    private static final class_2960 GUI_ICONS_LOCATION = new class_2960("textures/gui/icons.png");

    private final class_310 minecraft = class_310.method_1551();

    public boolean isHotbar = false;

    public VanillaLikeAttackIndicatorElement(@NotNull ElementBuilder<?, ?> builder) {
        super(builder);
    }

    /**
     * Renders the experience bar within this element's bounds.
     *
     * @param graphics The graphics context.
     * @param mouseX   The current mouse X-coordinate.
     * @param mouseY   The current mouse Y-coordinate.
     * @param partial  Partial ticks.
     */
    @Override
    public void render(@NotNull GuiGraphics graphics, int mouseX, int mouseY, float partial) {

        // Do nothing if the player or level is missing.
        if (this.minecraft.field_1724 == null || this.minecraft.field_1687 == null) {
            return;
        }

        // Get the absolute position and dimensions of this element.
        int elementX = this.getAbsoluteX();
        int elementY = this.getAbsoluteY();
        int elementWidth = this.getAbsoluteWidth();
        int elementHeight = this.getAbsoluteHeight();

        // Prepare the render state.
        RenderSystem.enableBlend();
        RenderingUtils.resetShaderColor(graphics);

        // Render the experience bar using the element's own bounds.
        this.renderAttackIndicator(graphics, this.isHotbar, elementX, elementY, elementWidth, elementHeight);

        RenderingUtils.resetShaderColor(graphics);

    }

    /**
     * Renders the attack indicator in either crosshair or hotbar mode, scaled to the specified dimensions.
     *
     * @param graphics the GuiGraphics object used for rendering
     * @param isHotbar whether to render the hotbar version (true) or crosshair version (false)
     * @param x the x position to render at
     * @param y the y position to render at
     * @param width the width to render the indicator at
     * @param height the height to render the indicator at
     */
    public void renderAttackIndicator(GuiGraphics graphics, boolean isHotbar, int x, int y, int width, int height) {

        float attackStrength = this.minecraft.field_1724.method_7261(0.0f);
        if (isEditor()) attackStrength = 0.5f;

        RenderSystem.enableBlend();
        graphics.setColor(1.0F, 1.0F, 1.0F, this.opacity);

        if (isHotbar) {
            // Render hotbar version (only shown when not fully charged)
            if (attackStrength < 1.0f) {
                // First render the background
                graphics.blit(GUI_ICONS_LOCATION, x, y, width, height, 0, 94, 18, 18, 256, 256);

                // Then render the foreground based on attack strength
                int fillHeight = (int)(height * attackStrength);
                if (fillHeight > 0) {
                    // Calculate the texture offset based on the fill height proportion
                    int textureFillHeight = (int)(18 * attackStrength);
                    graphics.blit(GUI_ICONS_LOCATION,
                            x, y + height - fillHeight,
                            width, fillHeight,
                            18, 112 - textureFillHeight,
                            18, textureFillHeight,
                            256, 256);
                }
            }
        } else {
            // Render crosshair version
            boolean readyToAttack = false;
            if (this.minecraft.field_1692 != null &&
                    this.minecraft.field_1692 instanceof class_1309 &&
                    attackStrength >= 1.0f) {
                readyToAttack = this.minecraft.field_1724.method_7279() > 5.0f;
                readyToAttack &= this.minecraft.field_1692.method_5805();
            }

            if (readyToAttack) {
                // Render the "ready to attack" indicator
                graphics.blit(GUI_ICONS_LOCATION, x, y, width, height, 68, 94, 16, 16, 256, 256);
            } else if (attackStrength < 1.0f) {
                // Determine a proper height for the progress bar (maintaining aspect ratio)
                int barHeight = height;
                if (width / height > 4) { // Original aspect ratio is 16:4 or 4:1
                    barHeight = Math.max(1, width / 4);
                }

                // First render the background bar
                graphics.blit(GUI_ICONS_LOCATION, x, y, width, barHeight, 36, 94, 16, 4, 256, 256);

                // Then render the foreground based on attack strength
                int fillWidth = (int)(width * attackStrength);
                if (fillWidth > 0) {
                    // The texture width is 17 for the fill bar
                    int textureWidth = (int)(17 * attackStrength);
                    graphics.blit(GUI_ICONS_LOCATION,
                            x, y,
                            fillWidth, barHeight,
                            52, 94,
                            textureWidth, 4,
                            256, 256);
                }
            }
        }

        graphics.setColor(1.0F, 1.0F, 1.0F, 1.0F);

    }

}
